/*
 * Path and directory definitions
 *
 * Derived from the mingw header written by Colin Peters.
 * Modified for Wine use by Jon Griffiths and Francois Gouget.
 * This file is in the public domain.
 */
#ifndef __WINE_DIRECT_H
#define __WINE_DIRECT_H

#ifndef __WINE_USE_MSVCRT
#define __WINE_USE_MSVCRT
#endif

#define CSI_MSVCRT_DIR_FUNCTIONS

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _WCHAR_T_DEFINED
#define _WCHAR_T_DEFINED
#ifndef __cplusplus
typedef unsigned short MSVCRT_wchar_t;
#endif
#endif

#ifndef _SIZE_T_DEFINED
typedef unsigned int size_t;
#define _SIZE_T_DEFINED
#endif

#ifndef _DISKFREE_T_DEFINED
#define _DISKFREE_T_DEFINED
struct _diskfree_t {
  unsigned int total_clusters;
  unsigned int avail_clusters;
  unsigned int sectors_per_cluster;
  unsigned int bytes_per_sector;
};
#endif /* _DISKFREE_T_DEFINED */

#ifdef CSI_MSVCRT_DIR_FUNCTIONS
int         MSVCRT(_chdir)(const char*);
#else
int         _chdir(const char*);
#endif
int         _chdrive(int);
#ifdef CSI_MSVCRT_DIR_FUNCTIONS
char*       MSVCRT(_getcwd)(char*,int);
#else
char*       _getcwd(char*,int);
#endif
char*       _getdcwd(int,char*,int);
int         _getdrive(void);
unsigned long _getdrives(void);
#ifdef CSI_MSVCRT_DIR_FUNCTIONS
int         MSVCRT(_mkdir)(const char*);
int         MSVCRT(_rmdir)(const char*);
#else
int         _mkdir(const char*);
int         _rmdir(const char*);
#endif

#ifndef _WDIRECT_DEFINED
#define _WDIRECT_DEFINED
int              _wchdir(const MSVCRT_wchar_t*);
MSVCRT_wchar_t* _wgetcwd(MSVCRT_wchar_t*,int);
MSVCRT_wchar_t* _wgetdcwd(int,MSVCRT_wchar_t*,int);
int              _wmkdir(const MSVCRT_wchar_t*);
int              _wrmdir(const MSVCRT_wchar_t*);
#endif /* _WDIRECT_DEFINED */

#ifdef __cplusplus
}
#endif


#ifndef CSI_MSVCRT_DIR_FUNCTIONS
static inline int chdir(const char* newdir) { return _chdir(newdir); }
static inline char* getcwd(char * buf, int size) { return _getcwd(buf, size); }
static inline int mkdir(const char* newdir) { return _mkdir(newdir); }
static inline int rmdir(const char* dir) { return _rmdir(dir); }
#else
#define getcwd  MSVCRT__getcwd
#define _getcwd MSVCRT__getcwd

#define chdir   MSVCRT__chdir
#define _chdir  MSVCRT__chdir

#define mkdir   MSVCRT__mkdir
#define _mkdir  MSVCRT__mkdir

#define rmdir   MSVCRT__rmdir
#define _rmdir  MSVCRT__rmdir
#endif /* CSI_MSVCRT_DIR_FUNCTIONS */

#endif /* __WINE_DIRECT_H */

