/*
 * Unicode definitions
 *
 * Derived from the mingw header written by Colin Peters.
 * Modified for Wine use by Jon Griffiths and Francois Gouget.
 * This file is in the public domain.
 */
#ifndef __WINE_WCHAR_H
#define __WINE_WCHAR_H
#ifndef __WINE_USE_MSVCRT
#define __WINE_USE_MSVCRT
#endif

#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _WCHAR_T_DEFINED
#define _WCHAR_T_DEFINED
#ifndef __cplusplus
typedef unsigned short MSVCRT_wchar_t;
#endif
#endif

#ifndef NULL
#ifdef __cplusplus
#define NULL  0
#else
#define NULL  ((void *)0)
#endif
#endif

#define WCHAR_MIN 0
#define WCHAR_MAX ((MSVCRT_wchar_t)-1)

typedef int mbstate_t;

#ifndef _SIZE_T_DEFINED
typedef unsigned int size_t;
#define _SIZE_T_DEFINED
#endif

#ifndef _WCTYPE_T_DEFINED
typedef unsigned short  wint_t;
typedef unsigned short  wctype_t;
#define _WCTYPE_T_DEFINED
#endif

#ifndef _MSC_VER
# ifndef __int64
#  define __int64 long long
# endif
#endif

#ifndef WEOF
#define WEOF        (wint_t)(0xFFFF)
#endif

#ifndef _FSIZE_T_DEFINED
typedef unsigned long _fsize_t;
#define _FSIZE_T_DEFINED
#endif

#ifndef _DEV_T_DEFINED
typedef unsigned int   _dev_t;
#define _DEV_T_DEFINED
#endif

#ifndef _INO_T_DEFINED
typedef unsigned short _ino_t;
#define _INO_T_DEFINED
#endif

#ifndef _OFF_T_DEFINED
typedef int _off_t;
#define _OFF_T_DEFINED
#endif

#ifndef _TIME_T_DEFINED
typedef long time_t;
#define _TIME_T_DEFINED
#endif

#ifndef _TM_DEFINED
#define _TM_DEFINED
struct tm {
    int tm_sec;
    int tm_min;
    int tm_hour;
    int tm_mday;
    int tm_mon;
    int tm_year;
    int tm_wday;
    int tm_yday;
    int tm_isdst;
};
#endif /* _TM_DEFINED */

#ifndef _FILE_DEFINED
#define _FILE_DEFINED
typedef struct _iobuf
{
  char* _ptr;
  int   _cnt;
  char* _base;
  int   _flag;
  int   _file;
  int   _charbuf;
  int   _bufsiz;
  char* _tmpfname;
} FILE;
#endif  /* _FILE_DEFINED */

#ifndef _WFINDDATA_T_DEFINED
#define _WFINDDATA_T_DEFINED

struct _wfinddata_t {
  unsigned attrib;
  time_t time_create;
  time_t time_access;
  time_t time_write;
  _fsize_t size;
  MSVCRT_wchar_t name[260];
};

struct _wfinddatai64_t {
  unsigned attrib;
  time_t time_create;
  time_t time_access;
  time_t time_write;
  __int64        size;
  MSVCRT_wchar_t name[260];
};

#endif /* _WFINDDATA_T_DEFINED */

#ifndef _STAT_DEFINED
#define _STAT_DEFINED

struct _stat {
  _dev_t st_dev;
  _ino_t st_ino;
  unsigned short st_mode;
  short          st_nlink;
  short          st_uid;
  short          st_gid;
  _dev_t st_rdev;
  _off_t st_size;
  time_t st_atime;
  time_t st_mtime;
  time_t st_ctime;
};

struct stat {
  _dev_t st_dev;
  _ino_t st_ino;
  unsigned short st_mode;
  short          st_nlink;
  short          st_uid;
  short          st_gid;
  _dev_t st_rdev;
  _off_t st_size;
  time_t st_atime;
  time_t st_mtime;
  time_t st_ctime;
};

struct _stati64 {
  _dev_t st_dev;
  _ino_t st_ino;
  unsigned short st_mode;
  short          st_nlink;
  short          st_uid;
  short          st_gid;
  _dev_t st_rdev;
  __int64        st_size;
  time_t st_atime;
  time_t st_mtime;
  time_t st_ctime;
};
#endif /* _STAT_DEFINED */

/* ASCII char classification table - binary compatible */
#define _UPPER        0x0001  /* C1_UPPER */
#define _LOWER        0x0002  /* C1_LOWER */
#define _DIGIT        0x0004  /* C1_DIGIT */
#define _SPACE        0x0008  /* C1_SPACE */
#define _PUNCT        0x0010  /* C1_PUNCT */
#define _CONTROL      0x0020  /* C1_CNTRL */
#define _BLANK        0x0040  /* C1_BLANK */
#define _HEX          0x0080  /* C1_XDIGIT */
#define _LEADBYTE     0x8000
#define _ALPHA       (0x0100|_UPPER|_LOWER)  /* (C1_ALPHA|_UPPER|_LOWER) */

#ifndef _WCTYPE_DEFINED
#define _WCTYPE_DEFINED
int is_wctype(wint_t,wctype_t);
int isleadbyte(int);
int iswalnum(wint_t);
int iswalpha(wint_t);
int iswascii(wint_t);
int iswcntrl(wint_t);
int iswctype(wint_t,wctype_t);
int iswdigit(wint_t);
int iswgraph(wint_t);
int iswlower(wint_t);
int iswprint(wint_t);
int iswpunct(wint_t);
int iswspace(wint_t);
int iswupper(wint_t);
int iswxdigit(wint_t);
MSVCRT_wchar_t towlower(MSVCRT_wchar_t);
MSVCRT_wchar_t towupper(MSVCRT_wchar_t);
#endif /* _WCTYPE_DEFINED */

#ifndef _WDIRECT_DEFINED
#define _WDIRECT_DEFINED
int              _wchdir(const MSVCRT_wchar_t*);
MSVCRT_wchar_t* _wgetcwd(MSVCRT_wchar_t*,int);
MSVCRT_wchar_t* _wgetdcwd(int,MSVCRT_wchar_t*,int);
int              _wmkdir(const MSVCRT_wchar_t*);
int              _wrmdir(const MSVCRT_wchar_t*);
#endif /* _WDIRECT_DEFINED */

#ifndef _WIO_DEFINED
#define _WIO_DEFINED
int         _waccess(const MSVCRT_wchar_t*,int);
int         _wchmod(const MSVCRT_wchar_t*,int);
int         _wcreat(const MSVCRT_wchar_t*,int);
long        _wfindfirst(const MSVCRT_wchar_t*,struct _wfinddata_t*);
long        _wfindfirsti64(const MSVCRT_wchar_t*, struct _wfinddatai64_t*);
int         _wfindnext(long,struct _wfinddata_t*);
int         _wfindnexti64(long, struct _wfinddatai64_t*);
MSVCRT_wchar_t*_wmktemp(MSVCRT_wchar_t*);
int         _wopen(const MSVCRT_wchar_t*,int,...);
int         _wrename(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
int         _wsopen(const MSVCRT_wchar_t*,int,int,...);
int         _wunlink(const MSVCRT_wchar_t*);
#endif /* _WIO_DEFINED */

#ifndef _WLOCALE_DEFINED
#define _WLOCALE_DEFINED
MSVCRT_wchar_t* _wsetlocale(int,const MSVCRT_wchar_t*);
#endif /* _WLOCALE_DEFINED */

#ifndef _WPROCESS_DEFINED
#define _WPROCESS_DEFINED
int         _wexecl(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wexecle(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wexeclp(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wexeclpe(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wexecv(const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *);
int         _wexecve(const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *,const MSVCRT_wchar_t* const *);
int         _wexecvp(const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *);
int         _wexecvpe(const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *,const MSVCRT_wchar_t* const *);
int         _wspawnl(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wspawnle(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wspawnlp(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wspawnlpe(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int         _wspawnv(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *);
int         _wspawnve(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *,const MSVCRT_wchar_t* const *);
int         _wspawnvp(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *);
int         _wspawnvpe(int,const MSVCRT_wchar_t*,const MSVCRT_wchar_t* const *,const MSVCRT_wchar_t* const *);
int         _wsystem(const MSVCRT_wchar_t*);
#endif /* _WPROCESS_DEFINED */

#ifndef _WSTAT_DEFINED
#define _WSTAT_DEFINED
int _wstat(const MSVCRT_wchar_t*,struct _stat*);
int _wstati64(const MSVCRT_wchar_t*,struct _stati64*);
#endif /* _WSTAT_DEFINED */

#ifndef _WSTDIO_DEFINED
#define _WSTDIO_DEFINED
wint_t  _fgetwchar(void);
wint_t  _fputwchar(wint_t);
MSVCRT_wchar_t*_getws(MSVCRT_wchar_t*);
int             _putws(const MSVCRT_wchar_t*);
int             _snwprintf(MSVCRT_wchar_t*,size_t,const MSVCRT_wchar_t*,...);
int             _vsnwprintf(MSVCRT_wchar_t*,size_t,const MSVCRT_wchar_t*,va_list);
FILE*   _wfdopen(int,const MSVCRT_wchar_t*);
FILE*   _wfopen(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
FILE*   _wfreopen(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,FILE*);
FILE*   _wfsopen(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,int);
void            _wperror(const MSVCRT_wchar_t*);
FILE*   _wpopen(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
int             _wremove(const MSVCRT_wchar_t*);
MSVCRT_wchar_t*_wtempnam(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*_wtmpnam(MSVCRT_wchar_t*);

wint_t  fgetwc(FILE*);
MSVCRT_wchar_t*fgetws(MSVCRT_wchar_t*,int,FILE*);
wint_t  fputwc(wint_t,FILE*);
int             fputws(const MSVCRT_wchar_t*,FILE*);
int             fwprintf(FILE*,const MSVCRT_wchar_t*,...);
int             fputws(const MSVCRT_wchar_t*,FILE*);
int             fwscanf(FILE*,const MSVCRT_wchar_t*,...);
wint_t  getwc(FILE*);
wint_t  getwchar(void);
MSVCRT_wchar_t*getws(MSVCRT_wchar_t*);
wint_t  putwc(wint_t,FILE*);
wint_t  putwchar(wint_t);
int             putws(const MSVCRT_wchar_t*);
int             swprintf(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
int             swscanf(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,...);
wint_t  ungetwc(wint_t,FILE*);
int             vfwprintf(FILE*,const MSVCRT_wchar_t*,va_list);
int             vswprintf(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,va_list);
int             vwprintf(const MSVCRT_wchar_t*,va_list);
int             wprintf(const MSVCRT_wchar_t*,...);
int             wscanf(const MSVCRT_wchar_t*,...);
#endif /* _WSTDIO_DEFINED */

#ifndef _WSTDLIB_DEFINED
#define _WSTDLIB_DEFINED
MSVCRT_wchar_t*_itow(int,MSVCRT_wchar_t*,int);
MSVCRT_wchar_t*_i64tow(__int64,MSVCRT_wchar_t*,int);
MSVCRT_wchar_t*_ltow(long,MSVCRT_wchar_t*,int);
MSVCRT_wchar_t*_ui64tow(unsigned __int64,MSVCRT_wchar_t*,int);
MSVCRT_wchar_t*_ultow(unsigned long,MSVCRT_wchar_t*,int);
MSVCRT_wchar_t*_wfullpath(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,size_t);
MSVCRT_wchar_t*_wgetenv(const MSVCRT_wchar_t*);
void            _wmakepath(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
void            _wperror(const MSVCRT_wchar_t*);
int             _wputenv(const MSVCRT_wchar_t*);
void            _wsearchenv(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,MSVCRT_wchar_t*);
void            _wsplitpath(const MSVCRT_wchar_t*,MSVCRT_wchar_t*,MSVCRT_wchar_t*,MSVCRT_wchar_t*,MSVCRT_wchar_t*);
int             _wsystem(const MSVCRT_wchar_t*);
int             _wtoi(const MSVCRT_wchar_t*);
__int64         _wtoi64(const MSVCRT_wchar_t*);
long            _wtol(const MSVCRT_wchar_t*);
float           _wtof(const MSVCRT_wchar_t*);

size_t mbstowcs(MSVCRT_wchar_t*,const char*,size_t);
int            mbtowc(MSVCRT_wchar_t*,const char*,size_t);
double         wcstod(const MSVCRT_wchar_t*,MSVCRT_wchar_t**);
long           wcstol(const MSVCRT_wchar_t*,MSVCRT_wchar_t**,int);
size_t wcstombs(char*,const MSVCRT_wchar_t*,size_t);
unsigned long  wcstoul(const MSVCRT_wchar_t*,MSVCRT_wchar_t**,int);
int            wctomb(char*,MSVCRT_wchar_t);
#endif /* _WSTDLIB_DEFINED */

#ifndef _WSTRING_DEFINED
#define _WSTRING_DEFINED
MSVCRT_wchar_t*_wcsdup(const MSVCRT_wchar_t*);
int             _wcsicmp(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
int             _wcsicoll(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*_wcslwr(MSVCRT_wchar_t*);
int             _wcsnicmp(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,size_t);
MSVCRT_wchar_t*_wcsnset(MSVCRT_wchar_t*,MSVCRT_wchar_t,size_t);
MSVCRT_wchar_t*_wcsrev(MSVCRT_wchar_t*);
MSVCRT_wchar_t*_wcsset(MSVCRT_wchar_t*,MSVCRT_wchar_t);
MSVCRT_wchar_t*_wcsupr(MSVCRT_wchar_t*);

MSVCRT_wchar_t*wcscat(MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*wcschr(const MSVCRT_wchar_t*,MSVCRT_wchar_t);
int             wcscmp(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
int             wcscoll(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*wcscpy(MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
size_t  wcscspn(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
size_t  wcslen(const MSVCRT_wchar_t*);
MSVCRT_wchar_t*wcsncat(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,size_t);
int             wcsncmp(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*,size_t);
MSVCRT_wchar_t*wcsncpy(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,size_t);
MSVCRT_wchar_t*wcspbrk(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*wcsrchr(const MSVCRT_wchar_t*,MSVCRT_wchar_t wcFor);
size_t  wcsspn(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*wcsstr(const MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
MSVCRT_wchar_t*wcstok(MSVCRT_wchar_t*,const MSVCRT_wchar_t*);
size_t  wcsxfrm(MSVCRT_wchar_t*,const MSVCRT_wchar_t*,size_t);
#endif /* _WSTRING_DEFINED */

#ifndef _WTIME_DEFINED
#define _WTIME_DEFINED
MSVCRT_wchar_t*_wasctime(const struct tm*);
size_t  wcsftime(MSVCRT_wchar_t*,size_t,const MSVCRT_wchar_t*,const struct tm*);
MSVCRT_wchar_t*_wctime(const time_t*);
MSVCRT_wchar_t*_wstrdate(MSVCRT_wchar_t*);
MSVCRT_wchar_t*_wstrtime(MSVCRT_wchar_t*);
#endif /* _WTIME_DEFINED */

MSVCRT_wchar_t btowc(int);
size_t  mbrlen(const char *,size_t,mbstate_t*);
size_t  mbrtowc(MSVCRT_wchar_t*,const char*,size_t,mbstate_t*);
size_t  mbsrtowcs(MSVCRT_wchar_t*,const char**,size_t,mbstate_t*);
size_t  wcrtomb(char*,MSVCRT_wchar_t,mbstate_t*);
size_t  wcsrtombs(char*,const MSVCRT_wchar_t**,size_t,mbstate_t*);
int             wctob(wint_t);

#ifdef __cplusplus
}
#endif

#endif /* __WINE_WCHAR_H */

